@@ -2,25 +2,36 @@ from __future__ import annotations
 
 import json
 import os
-from math import ceil
 from typing import TYPE_CHECKING
 
 import numpy as np
 
+from auto_editor.objs.edit import (
+    Audio,
+    Motion,
+    Pixeldiff,
+    audio_builder,
+    motion_builder,
+    pixeldiff_builder,
+)
+from auto_editor.objs.util import _Vars, parse_dataclass
+from auto_editor.utils.func import boolop
 from auto_editor.wavfile import read
 
 if TYPE_CHECKING:
     from fractions import Fraction
+    from typing import Any
 
     from numpy.typing import NDArray
 
     from auto_editor.ffwrapper import FileInfo
+    from auto_editor.interpreter import FileSetup
     from auto_editor.output import Ensure
     from auto_editor.utils.bar import Bar
     from auto_editor.utils.log import Log
 
 
-def link_nodes(*nodes):
+def link_nodes(*nodes: Any) -> None:
     for c, n in zip(nodes, nodes[1:]):
         c.link_to(n)
 
@@ -30,13 +41,13 @@ def to_threshold(arr: np.ndarray, t: int | float) -> NDArray[np.bool_]:
 
 
 def get_media_length(
-    ensure: Ensure, inp: FileInfo, tb: Fraction, temp: str, log: Log
+    ensure: Ensure, src: FileInfo, tb: Fraction, temp: str, log: Log
 ) -> int:
-    if inp.audios:
-        if (arr := read_cache(inp, tb, "audio", {"stream": 0}, temp)) is not None:
+    if src.audios:
+        if (arr := read_cache(src, tb, "audio", {"stream": 0}, temp)) is not None:
             return len(arr)
 
-        sr, samples = read(ensure.audio(inp.path, inp.index, stream=0))
+        sr, samples = read(ensure.audio(src.path, src.label, stream=0))
         samp_count = len(samples)
         del samples
 
@@ -51,7 +62,7 @@ def get_media_length(
 
     av.logging.set_level(av.logging.PANIC)
 
-    with av.open(inp.path, "r") as cn:
+    with av.open(src.path) as cn:
         if len(cn.streams.video) < 1:
             log.error("Could not get media duration")
 
@@ -63,18 +74,18 @@ def get_media_length(
 
 
 def get_all(
-    ensure: Ensure, inp: FileInfo, tb: Fraction, temp: str, log: Log
+    ensure: Ensure, src: FileInfo, tb: Fraction, temp: str, log: Log
 ) -> NDArray[np.bool_]:
-    return np.zeros(get_media_length(ensure, inp, tb, temp, log), dtype=np.bool_)
+    return np.zeros(get_media_length(ensure, src, tb, temp, log), dtype=np.bool_)
 
 
 def get_none(
-    ensure: Ensure, inp: FileInfo, tb: Fraction, temp: str, log: Log
+    ensure: Ensure, src: FileInfo, tb: Fraction, temp: str, log: Log
 ) -> NDArray[np.bool_]:
-    return np.ones(get_media_length(ensure, inp, tb, temp, log), dtype=np.bool_)
+    return np.ones(get_media_length(ensure, src, tb, temp, log), dtype=np.bool_)
 
 
-def _dict_tag(tag: str, tb: Fraction, obj) -> tuple[str, dict]:
+def _dict_tag(tag: str, tb: Fraction, obj: Any) -> tuple[str, dict]:
     if isinstance(obj, dict):
         obj_dict = obj.copy()
     else:
@@ -91,7 +102,7 @@ def _dict_tag(tag: str, tb: Fraction, obj) -> tuple[str, dict]:
 
 
 def read_cache(
-    inp: FileInfo, tb: Fraction, tag: str, obj, temp: str
+    src: FileInfo, tb: Fraction, tag: str, obj: Any, temp: str
 ) -> None | np.ndarray:
     from auto_editor import version
 
@@ -103,19 +114,19 @@ def read_cache(
     with open(workfile) as file:
         cache = json.load(file)
 
-    if inp.path not in cache:
+    if src.path not in cache:
         return None
 
     key, obj_dict = _dict_tag(tag, tb, obj)
 
-    if key not in (root := cache[inp.path]):
+    if key not in (root := cache[src.path]):
         return None
 
     return np.asarray(root[key]["arr"], dtype=root[key]["type"])
 
 
 def cache(
-    tag: str, tb: Fraction, obj, arr: np.ndarray, inp: FileInfo, temp: str
+    tag: str, tb: Fraction, obj: Any, arr: np.ndarray, src: FileInfo, temp: str
 ) -> np.ndarray:
     from auto_editor import version
 
@@ -137,10 +148,10 @@ def cache(
         "arr": arr.tolist(),
     }
 
-    if inp.path in json_object:
-        json_object[inp.path][key] = entry
+    if src.path in json_object:
+        json_object[src.path][key] = entry
     else:
-        json_object[inp.path] = {key: entry}
+        json_object[src.path] = {key: entry}
 
     with open(os.path.join(workdur, "cache.json"), "w") as file:
         file.write(json.dumps(json_object))
@@ -150,7 +161,7 @@ def cache(
 
 def audio_levels(
     ensure: Ensure,
-    inp: FileInfo,
+    src: FileInfo,
     s: int,
     tb: Fraction,
     bar: Bar,
@@ -159,15 +170,15 @@ def audio_levels(
     log: Log,
 ) -> NDArray[np.float_]:
 
-    if s > len(inp.audios) - 1:
+    if s > len(src.audios) - 1:
         if strict:
             log.error(f"Audio stream '{s}' does not exist.")
-        return get_all(ensure, inp, tb, temp, log)
+        return np.zeros(get_media_length(ensure, src, tb, temp, log), dtype=np.float_)
 
-    if (arr := read_cache(inp, tb, "audio", {"stream": s}, temp)) is not None:
+    if (arr := read_cache(src, tb, "audio", {"stream": s}, temp)) is not None:
         return arr
 
-    sr, samples = read(ensure.audio(inp.path, inp.index, s))
+    sr, samples = read(ensure.audio(src.path, src.label, s))
 
     def get_max_volume(s: np.ndarray) -> float:
         return max(float(np.max(s)), -float(np.min(s)))
@@ -178,8 +189,8 @@ def audio_levels(
     samp_count = samples.shape[0]
     samp_per_ticks = sr / tb
 
-    audio_ticks = ceil(samp_count / samp_per_ticks)
-    log.debug(f"Audio Length: {audio_ticks}")
+    audio_ticks = int(samp_count / samp_per_ticks)
+    log.debug(f"analyze: Audio Length: {audio_ticks}")
     log.debug(f"... no rounding: {float(samp_count / samp_per_ticks)}")
 
     bar.start(audio_ticks, "Analyzing audio volume")
@@ -200,13 +211,13 @@ def audio_levels(
         threshold_list[i] = get_max_volume(samples[start:end]) / max_volume
 
     bar.end()
-    return cache("audio", tb, {"stream": s}, threshold_list, inp, temp)
+    return cache("audio", tb, {"stream": s}, threshold_list, src, temp)
 
 
 def motion_levels(
     ensure: Ensure,
-    inp: FileInfo,
-    mobj,
+    src: FileInfo,
+    mobj: Any,
     tb: Fraction,
     bar: Bar,
     strict: bool,
@@ -218,26 +229,31 @@ def motion_levels(
 
     av.logging.set_level(av.logging.PANIC)
 
-    if mobj.stream >= len(inp.videos):
+    if mobj.stream >= len(src.videos):
         if not strict:
-            return get_all(ensure, inp, tb, temp, log)
+            return np.zeros(
+                get_media_length(ensure, src, tb, temp, log), dtype=np.float_
+            )
         log.error(f"Video stream '{mobj.stream}' does not exist.")
 
-    if (arr := read_cache(inp, tb, "motion", mobj, temp)) is not None:
+    if (arr := read_cache(src, tb, "motion", mobj, temp)) is not None:
         return arr
 
-    container = av.open(inp.path, "r")
+    container = av.open(src.path, "r")
 
     stream = container.streams.video[mobj.stream]
     stream.thread_type = "AUTO"
 
-    inaccurate_dur = int(stream.duration * stream.time_base * stream.rate)
+    if stream.duration is None:
+        inaccurate_dur = 1
+    else:
+        inaccurate_dur = int(stream.duration * stream.time_base * stream.average_rate)
 
     bar.start(inaccurate_dur, "Analyzing motion")
 
     prev_image = None
     image = None
-    total_pixels = None
+    total_pixels = src.videos[0].width * src.videos[0].height
     index = 0
 
     graph = av.filter.Graph()
@@ -267,9 +283,6 @@ def motion_levels(
 
         image = frame.to_image().convert("L")
 
-        if total_pixels is None:
-            total_pixels = image.size[0] * image.size[1]
-
         if mobj.blur > 0:
             image = image.filter(ImageFilter.GaussianBlur(radius=mobj.blur))
 
@@ -282,13 +295,13 @@ def motion_levels(
     result = threshold_list[:index]
     del threshold_list
 
-    return cache("motion", tb, mobj, result, inp, temp)
+    return cache("motion", tb, mobj, result, src, temp)
 
 
 def pixeldiff_levels(
     ensure: Ensure,
-    inp: FileInfo,
-    pobj,
+    src: FileInfo,
+    pobj: Any,
     tb: Fraction,
     bar: Bar,
     strict: bool,
@@ -300,20 +313,25 @@ def pixeldiff_levels(
 
     av.logging.set_level(av.logging.PANIC)
 
-    if pobj.stream >= len(inp.videos):
+    if pobj.stream >= len(src.videos):
         if not strict:
-            return get_all(ensure, inp, tb, temp, log)
+            return np.zeros(
+                get_media_length(ensure, src, tb, temp, log), dtype=np.uint64
+            )
         log.error(f"Video stream '{pobj.stream}' does not exist.")
 
-    if (arr := read_cache(inp, tb, "pixeldiff", pobj, temp)) is not None:
+    if (arr := read_cache(src, tb, "pixeldiff", pobj, temp)) is not None:
         return arr
 
-    container = av.open(inp.path, "r")
+    container = av.open(src.path, "r")
 
     stream = container.streams.video[pobj.stream]
     stream.thread_type = "AUTO"
 
-    inaccurate_dur = int(stream.duration * stream.time_base * stream.rate)
+    if stream.duration is None:
+        inaccurate_dur = 1
+    else:
+        inaccurate_dur = int(stream.duration * stream.time_base * stream.average_rate)
 
     bar.start(inaccurate_dur, "Analyzing pixel diffs")
 
@@ -346,22 +364,79 @@ def pixeldiff_levels(
     result = threshold_list[:index]
     del threshold_list
 
-    return cache("pixeldiff", tb, pobj, result, inp, temp)
+    return cache("pixeldiff", tb, pobj, result, src, temp)
 
 
-def random_levels(
-    ensure: Ensure, inp: FileInfo, robj, timebase: Fraction, temp: str, log: Log
-) -> NDArray[np.float_]:
-    import random
+def edit_method(val: str, filesetup: FileSetup) -> NDArray[np.bool_]:
+    src = filesetup.src
+    tb = filesetup.tb
+    ensure = filesetup.ensure
+    strict = filesetup.strict
 
-    if robj.seed == -1:
-        robj.seed = random.randint(0, 2147483647)
+    bar = filesetup.bar
+    temp = filesetup.temp
+    log = filesetup.log
 
-    random.seed(robj.seed)
-    log.debug(f"Seed: {robj.seed}")
+    METHODS = ("audio", "motion", "pixeldiff", "none", "all")
 
-    arr = [
-        random.random()
-        for _ in range(get_media_length(ensure, inp, timebase, temp, log))
-    ]
-    return np.array(arr, dtype=np.float_)
+    if ":" in val:
+        method, attrs = val.split(":")
+        if method not in METHODS:
+            log.error(f"'{method}' not allowed to have attributes")
+    else:
+        method, attrs = val, ""
+
+    if method == "none":
+        return get_none(ensure, src, tb, temp, log)
+
+    if method == "all":
+        return get_all(ensure, src, tb, temp, log)
+
+    if method == "audio":
+        aobj = parse_dataclass(attrs, (Audio, audio_builder), log)
+        s = aobj.stream
+        if s == "all":
+            total_list: NDArray[np.bool_] | None = None
+            for s in range(len(src.audios)):
+                audio_list = to_threshold(
+                    audio_levels(ensure, src, s, tb, bar, strict, temp, log),
+                    aobj.threshold,
+                )
+                if total_list is None:
+                    total_list = audio_list
+                else:
+                    total_list = boolop(total_list, audio_list, np.logical_or)
+            if total_list is None:
+                if strict:
+                    log.error("Input has no audio streams.")
+                stream_data = get_all(ensure, src, tb, temp, log)
+            else:
+                stream_data = total_list
+        else:
+            stream_data = to_threshold(
+                audio_levels(ensure, src, s, tb, bar, strict, temp, log),
+                aobj.threshold,
+            )
+
+        return stream_data
+
+    if method == "motion":
+        if src.videos:
+            _vars: _Vars = {"width": src.videos[0].width}
+        else:
+            _vars = {"width": 1}
+
+        mobj = parse_dataclass(attrs, (Motion, motion_builder), log, _vars)
+        return to_threshold(
+            motion_levels(ensure, src, mobj, tb, bar, strict, temp, log),
+            mobj.threshold,
+        )
+
+    if method == "pixeldiff":
+        pobj = parse_dataclass(attrs, (Pixeldiff, pixeldiff_builder), log)
+        return to_threshold(
+            pixeldiff_levels(ensure, src, pobj, tb, bar, strict, temp, log),
+            pobj.threshold,
+        )
+
+    raise ValueError("Unreachable")
