@@ -12,14 +12,14 @@ from auto_editor.utils.types import Args
 class Ensure:
     __slots__ = ("_ffmpeg", "_sr", "temp", "log")
 
-    def __init__(self, ffmpeg: FFmpeg, sr: int, temp: str, log: Log) -> None:
+    def __init__(self, ffmpeg: FFmpeg, sr: int, temp: str, log: Log):
         self._ffmpeg = ffmpeg
         self._sr = sr
         self.temp = temp
         self.log = log
 
-    def audio(self, path: str, index: int, stream: int) -> str:
-        out_path = os.path.join(self.temp, f"{index}-{stream}.wav")
+    def audio(self, path: str, label: str, stream: int) -> str:
+        out_path = os.path.join(self.temp, f"{label}-{stream}.wav")
 
         if not os.path.isfile(out_path):
             self.log.conwrite("Extracting audio")
@@ -32,14 +32,12 @@ class Ensure:
 
 
 def _ffset(cmd: list[str], option: str, value: str | None) -> list[str]:
-    if value is None or value == "unset":
+    if value is None or value == "unset" or value == "reserved":
         return cmd
     return cmd + [option] + [value]
 
 
-def video_quality(
-    cmd: list[str], args: Args, inp: FileInfo, ctr: Container
-) -> list[str]:
+def video_quality(cmd: list[str], args: Args, ctr: Container) -> list[str]:
     cmd = _ffset(cmd, "-b:v", args.video_bitrate)
     cmd.extend(["-c:v", args.video_codec])
     cmd = _ffset(cmd, "-qscale:v", args.video_quality_scale)
@@ -51,23 +49,24 @@ def mux_quality_media(
     ffmpeg: FFmpeg,
     visual_output: list[tuple[bool, str]],
     audio_output: list[str],
+    sub_output: list[str],
     apply_v: bool,
     ctr: Container,
     output_path: str,
     tb: Fraction,
     args: Args,
-    inp: FileInfo,
+    src: FileInfo,
     temp: str,
     log: Log,
 ) -> None:
 
-    s_tracks = 0 if not ctr.allow_subtitle else len(inp.subtitles)
-    a_tracks = 0 if not ctr.allow_audio else len(audio_output)
-    v_tracks = 0 if not ctr.allow_video else len(visual_output)
+    v_tracks = len(visual_output)
+    a_tracks = len(audio_output)
+    s_tracks = len(sub_output)
 
-    cmd = ["-hide_banner", "-y", "-i", inp.path]
+    cmd = ["-hide_banner", "-y", "-i", src.path]
 
-    same_container = inp.ext == os.path.splitext(output_path)[1]
+    same_container = src.ext == os.path.splitext(output_path)[1]
 
     for is_video, path in visual_output:
         if is_video or ctr.allow_image:
@@ -101,13 +100,10 @@ def mux_quality_media(
                 new_a_file = audio_output[0]
             cmd.extend(["-i", new_a_file])
 
-    if s_tracks > 0:
-        for s, sub in enumerate(inp.subtitles):
-            cmd.extend(["-i", os.path.join(temp, f"new{s}s.{sub.ext}")])
+    for subfile in sub_output:
+        cmd.extend(["-i", subfile])
 
-    total_streams = v_tracks + s_tracks + a_tracks
-
-    for i in range(total_streams):
+    for i in range(v_tracks + s_tracks + a_tracks):
         cmd.extend(["-map", f"{i+1}:0"])
 
     cmd.extend(["-map_metadata", "0"])
@@ -116,7 +112,7 @@ def mux_quality_media(
     for is_video, path in visual_output:
         if is_video:
             if apply_v:
-                cmd = video_quality(cmd, args, inp, ctr)
+                cmd = video_quality(cmd, args, ctr)
             else:
                 # Real video is only allowed on track 0
                 cmd.extend(["-c:v:0", "copy"])
@@ -132,24 +128,24 @@ def mux_quality_media(
         track += 1
     del track
 
-    for i, vstream in enumerate(inp.videos):
+    for i, vstream in enumerate(src.videos):
         if i > v_tracks:
             break
         if vstream.lang is not None:
             cmd.extend([f"-metadata:s:v:{i}", f"language={vstream.lang}"])
-    for i, astream in enumerate(inp.audios):
+    for i, astream in enumerate(src.audios):
         if i > a_tracks:
             break
         if astream.lang is not None:
             cmd.extend([f"-metadata:s:a:{i}", f"language={astream.lang}"])
-    for i, sstream in enumerate(inp.subtitles):
+    for i, sstream in enumerate(src.subtitles):
         if i > s_tracks:
             break
         if sstream.lang is not None:
             cmd.extend([f"-metadata:s:s:{i}", f"language={sstream.lang}"])
 
     if s_tracks > 0:
-        scodec = inp.subtitles[0].codec
+        scodec = src.subtitles[0].codec
         if same_container:
             cmd.extend(["-c:s", scodec])
         elif ctr.scodecs is not None:
@@ -162,10 +158,10 @@ def mux_quality_media(
         cmd = _ffset(cmd, "-b:a", args.audio_bitrate)
 
     if same_container and v_tracks > 0:
-        cmd = _ffset(cmd, "-color_range", inp.videos[0].color_range)
-        cmd = _ffset(cmd, "-colorspace", inp.videos[0].color_space)
-        cmd = _ffset(cmd, "-color_primaries", inp.videos[0].color_primaries)
-        cmd = _ffset(cmd, "-color_trc", inp.videos[0].color_transfer)
+        cmd = _ffset(cmd, "-color_range", src.videos[0].color_range)
+        cmd = _ffset(cmd, "-colorspace", src.videos[0].color_space)
+        cmd = _ffset(cmd, "-color_primaries", src.videos[0].color_primaries)
+        cmd = _ffset(cmd, "-color_trc", src.videos[0].color_transfer)
 
     if args.extras is not None:
         cmd.extend(args.extras.split(" "))
