@@ -21,15 +21,15 @@ def desc_options(parser: ArgumentParser) -> ArgumentParser:
     return parser
 
 
-def main(sys_args=sys.argv[1:]) -> None:
+def main(sys_args: list[str]) -> None:
     args = desc_options(ArgumentParser("desc")).parse_args(DescArgs, sys_args)
-    for i, istr in enumerate(args.input):
-        inp = FileInfo(i, istr, FFmpeg(args.ffmpeg_location, debug=False), Log())
-        if inp.description is not None:
-            sys.stdout.write(f"\n{inp.description}\n\n")
+    for path in args.input:
+        src = FileInfo(path, FFmpeg(args.ffmpeg_location, debug=False), Log())
+        if src.description is not None:
+            sys.stdout.write(f"\n{src.description}\n\n")
         else:
             sys.stdout.write("\nNo description.\n\n")
 
 
 if __name__ == "__main__":
-    main()
+    main(sys.argv[1:])
