@@ -72,7 +72,7 @@ def grep_file(
     media_file: str,
     add_prefix: bool,
     ffmpeg: FFmpeg,
-    args: type[GrepArgs],
+    args: GrepArgs,
     log: Log,
     TEMP: str,
 ) -> None:
@@ -135,7 +135,7 @@ def grep_file(
         print(prefix + str(count))
 
 
-def main(sys_args=sys.argv[1:]) -> None:
+def main(sys_args: list[str]) -> None:
     args = grep_options(ArgumentParser("grep")).parse_args(GrepArgs, sys_args)
     ffmpeg = FFmpeg(args.ffmpeg_location, args.my_ffmpeg, debug=False)
 
@@ -161,10 +161,10 @@ def main(sys_args=sys.argv[1:]) -> None:
         elif os.path.isfile(path):
             grep_file(path, add_prefix, ffmpeg, args, log, TEMP)
         else:
-            log.error(f"{path}: File does not exist.")
+            log.nofile(path)
 
     log.cleanup()
 
 
 if __name__ == "__main__":
-    main()
+    main(sys.argv[1:])
