@@ -81,10 +81,12 @@ class MediaJson(TypedDict, total=False):
     audio: list[AudioJson]
     subtitle: list[SubtitleJson]
     container: ContainerJson
-    media: Literal["invalid"]
+    type: Literal["media", "timeline", "unknown"]
+    version: Literal["v1", "v2"]
+    clips: int
 
 
-def main(sys_args=sys.argv[1:]) -> None:
+def main(sys_args: list[str]) -> None:
     args = info_options(ArgumentParser("info")).parse_args(InfoArgs, sys_args)
 
     ffmpeg = FFmpeg(args.ffmpeg_location, args.my_ffmpeg, False)
@@ -92,28 +94,46 @@ def main(sys_args=sys.argv[1:]) -> None:
 
     file_info: dict[str, MediaJson] = {}
 
-    for i, file in enumerate(args.input):
+    for file in args.input:
         if not os.path.isfile(file):
-            Log().error(f"Could not find file: {file}")
+            log.nofile(file)
 
-        inp = FileInfo(i, file, ffmpeg, log)
+        ext = os.path.splitext(file)[1]
+        if ext == ".json":
+            from auto_editor.formats.json import read_json
 
-        if len(inp.videos) + len(inp.audios) + len(inp.subtitles) == 0:
-            file_info[file] = {"media": "invalid"}
+            tl = read_json(file, ffmpeg, log)
+            file_info[file] = {"type": "timeline"}
+            file_info[file]["version"] = "v2" if tl.chunks is None else "v1"
+
+            clip_lens = [clip.dur / clip.speed for clip in tl.a[0]]
+            file_info[file]["clips"] = len(clip_lens)
+
+            continue
+
+        if ext in (".xml", ".fcpxml", ".mlt"):
+            file_info[file] = {"type": "timeline"}
+            continue
+
+        src = FileInfo(file, ffmpeg, log)
+
+        if len(src.videos) + len(src.audios) + len(src.subtitles) == 0:
+            file_info[file] = {"type": "unknown"}
             continue
 
         file_info[file] = {
+            "type": "media",
             "video": [],
             "audio": [],
             "subtitle": [],
             "container": {
-                "duration": inp.duration,
-                "bitrate": inp.bitrate,
+                "duration": src.duration,
+                "bitrate": src.bitrate,
                 "fps_mode": None,
             },
         }
 
-        for track, v in enumerate(inp.videos):
+        for track, v in enumerate(src.videos):
             w, h = v.width, v.height
 
             vid: VideoJson = {
@@ -134,7 +154,7 @@ def main(sys_args=sys.argv[1:]) -> None:
             }
             file_info[file]["video"].append(vid)
 
-        for track, a in enumerate(inp.audios):
+        for track, a in enumerate(src.audios):
             aud: AudioJson = {
                 "codec": a.codec,
                 "samplerate": a.samplerate,
@@ -144,7 +164,7 @@ def main(sys_args=sys.argv[1:]) -> None:
             }
             file_info[file]["audio"].append(aud)
 
-        for track, s_stream in enumerate(inp.subtitles):
+        for track, s_stream in enumerate(src.subtitles):
             sub: SubtitleJson = {"codec": s_stream.codec, "lang": s_stream.lang}
             file_info[file]["subtitle"].append(sub)
 
@@ -190,23 +210,22 @@ def main(sys_args=sys.argv[1:]) -> None:
     text = ""
     for name, info in file_info.items():
         text += f"{name}:\n"
-        if "media" in info:
-            text += " - invalid media\n\n"
-            continue
 
         for label, streams in info.items():
-            if isinstance(streams, dict):
+            if isinstance(streams, list):
+                text = stream_to_text(text, label, streams)
+                continue
+            elif isinstance(streams, dict):
                 text += " - container:\n"
                 for key, value in streams.items():
                     if value is not None:
                         text += f"   - {key}: {value}\n"
-            else:
-                assert isinstance(streams, list)
-                text = stream_to_text(text, label, streams)
+            elif label != "type" or streams != "media":
+                text += f" - {label}: {streams}\n"
         text += "\n"
 
     sys.stdout.write(text)
 
 
 if __name__ == "__main__":
-    main()
+    main(sys.argv[1:])
