@@ -1,29 +1,19 @@
 from __future__ import annotations
 
 import sys
-import tempfile
 from dataclasses import dataclass, field
 from fractions import Fraction
 
 import numpy as np
 from numpy.typing import NDArray
 
-from auto_editor.analyze import (
-    audio_levels,
-    motion_levels,
-    pixeldiff_levels,
-    random_levels,
-)
+from auto_editor.analyze import audio_levels, motion_levels, pixeldiff_levels
 from auto_editor.ffwrapper import FFmpeg, FileInfo
-from auto_editor.method import (
-    audio_builder,
-    motion_builder,
-    pixeldiff_builder,
-    random_builder,
-)
-from auto_editor.objects import parse_dataclass
+from auto_editor.objs.edit import audio_builder, motion_builder, pixeldiff_builder
+from auto_editor.objs.util import _Vars, parse_dataclass
 from auto_editor.output import Ensure
 from auto_editor.utils.bar import Bar
+from auto_editor.utils.func import setup_tempdir
 from auto_editor.utils.log import Log
 from auto_editor.utils.types import frame_rate
 from auto_editor.vanparse import ArgumentParser
@@ -46,11 +36,6 @@ class Pixeldiff:
     stream: int
 
 
-@dataclass
-class Random:
-    seed: int
-
-
 @dataclass
 class LevelArgs:
     input: list[str] = field(default_factory=list)
@@ -94,59 +79,58 @@ def print_ints(arr: NDArray[np.uint64]) -> None:
         sys.stdout.write(f"{a}\n")
 
 
-def main(sys_args=sys.argv[1:]) -> None:
+def main(sys_args: list[str]) -> None:
     parser = levels_options(ArgumentParser("levels"))
     args = parser.parse_args(LevelArgs, sys_args)
 
     ffmpeg = FFmpeg(args.ffmpeg_location, args.my_ffmpeg, False)
 
     bar = Bar("none")
-    temp = tempfile.mkdtemp()
+    temp = setup_tempdir(None, Log())
     log = Log(temp=temp)
 
-    inputs = [FileInfo(i, istr, ffmpeg, log) for i, istr in enumerate(args.input)]
+    sources = {}
+    for i, path in enumerate(args.input):
+        sources[str(i)] = FileInfo(path, ffmpeg, log, str(i))
 
-    tb = inputs[0].get_fps() if args.timebase is None else args.timebase
-    ensure = Ensure(ffmpeg, inputs[0].get_samplerate(), temp, log)
+    assert "0" in sources
+    src = sources["0"]
+
+    tb = src.get_fps() if args.timebase is None else args.timebase
+    ensure = Ensure(ffmpeg, src.get_samplerate(), temp, log)
 
     strict = True
+    METHODS = ("audio", "motion", "pixeldiff")
 
-    METHOD_ATTRS_SEP = ":"
-    METHODS = ("audio", "motion", "pixeldiff", "random")
-
-    if METHOD_ATTRS_SEP in args.edit:
-        method, attrs = args.edit.split(METHOD_ATTRS_SEP)
+    if ":" in args.edit:
+        method, attrs = args.edit.split(":")
     else:
         method, attrs = args.edit, ""
 
     if method not in METHODS:
         log.error(f"Method: {method} not supported")
 
-    for inp in inputs:
-        if method == "random":
-            robj = parse_dataclass(attrs, Random, random_builder[1:], log)
-            print_floats(random_levels(ensure, inp, robj, tb, temp, log))
-
+    for src in sources.values():
         if method == "audio":
-            aobj = parse_dataclass(attrs, Audio, audio_builder[1:], log)
+            aobj = parse_dataclass(attrs, (Audio, audio_builder[1:]), log)
             print_floats(
-                audio_levels(ensure, inp, aobj.stream, tb, bar, strict, temp, log)
+                audio_levels(ensure, src, aobj.stream, tb, bar, strict, temp, log)
             )
 
         if method == "motion":
-            if inp.videos:
-                _vars = {"width": inp.videos[0].width}
+            if src.videos:
+                _vars: _Vars = {"width": src.videos[0].width}
             else:
                 _vars = {"width": 1}
-            mobj = parse_dataclass(attrs, Motion, motion_builder[1:], log, _vars)
-            print_floats(motion_levels(ensure, inp, mobj, tb, bar, strict, temp, log))
+            mobj = parse_dataclass(attrs, (Motion, motion_builder[1:]), log, _vars)
+            print_floats(motion_levels(ensure, src, mobj, tb, bar, strict, temp, log))
 
         if method == "pixeldiff":
-            pobj = parse_dataclass(attrs, Pixeldiff, pixeldiff_builder[1:], log)
-            print_ints(pixeldiff_levels(ensure, inp, pobj, tb, bar, strict, temp, log))
+            pobj = parse_dataclass(attrs, (Pixeldiff, pixeldiff_builder[1:]), log)
+            print_ints(pixeldiff_levels(ensure, src, pobj, tb, bar, strict, temp, log))
 
     log.cleanup()
 
 
 if __name__ == "__main__":
-    main()
+    main(sys.argv[1:])
