@@ -2,7 +2,6 @@
 from __future__ import annotations
 
 import json
-import logging
 import os
 import platform
 import shutil
@@ -13,12 +12,13 @@ from fractions import Fraction
 from time import perf_counter
 from typing import Callable
 
-import av
+import numpy as np
 
+from auto_editor.ffwrapper import FFmpeg, FileInfo
+from auto_editor.interpreter import ConsType, Interpreter, Lexer, MyError, Null, Parser
+from auto_editor.utils.log import Log
 from auto_editor.vanparse import ArgumentParser
 
-av.logging.set_level(av.logging.PANIC)
-
 
 @dataclass
 class TestArgs:
@@ -32,7 +32,7 @@ def test_options(parser):
     parser.add_required(
         "category",
         nargs=1,
-        choices=("cli", "sub", "api", "all"),
+        choices=("unit", "cli", "sub", "api", "all"),
         metavar="category [options]",
     )
     return parser
@@ -44,6 +44,15 @@ def pipe_to_console(cmd: list[str]) -> tuple[int, str, str]:
     return process.returncode, stdout.decode("utf-8"), stderr.decode("utf-8")
 
 
+class Checker:
+    def __init__(self, ffmpeg: FFmpeg, log: Log):
+        self.ffmpeg = ffmpeg
+        self.log = log
+
+    def check(self, path: str) -> FileInfo:
+        return FileInfo(path, self.ffmpeg, self.log)
+
+
 class Runner:
     def __init__(self) -> None:
         if platform.system() == "Windows":
@@ -66,9 +75,15 @@ class Runner:
             root, ext = os.path.splitext(inputs[0])
 
             if "--export_as_json" in cmd:
-                output = f"{root}.json"
-            else:
-                output = f"{root}_ALTERED{ext}"
+                ext = ".json"
+            elif "-exp" in cmd:
+                ext = ".xml"
+            elif "-exf" in cmd:
+                ext = ".fcpxml"
+            elif "-exs" in cmd:
+                ext = ".mlt"
+
+            output = f"{root}_ALTERED{ext}"
 
         returncode, stdout, stderr = pipe_to_console(cmd)
         if returncode > 0:
@@ -93,58 +108,52 @@ class Runner:
             raise Exception("Program should not respond with code 0 but did!")
 
 
-class Tester:
-    def __init__(self) -> None:
-        pass
+def run_tests(tests: list[Callable], args: TestArgs) -> None:
+    def clean_all() -> None:
+        def clean(the_dir: str) -> None:
+            for item in os.listdir(the_dir):
+                if "_ALTERED" in item:
+                    os.remove(os.path.join(the_dir, item))
+                if item.endswith("_tracks"):
+                    shutil.rmtree(os.path.join(the_dir, item))
 
-    def run(self, tests: list[Callable], args: TestArgs) -> None:
-        def clean_all() -> None:
-            def clean(the_dir: str) -> None:
-                for item in os.listdir(the_dir):
-                    item = os.path.join(the_dir, item)
-                    if (
-                        "_ALTERED" in item
-                        or item.endswith(".xml")
-                        or item.endswith(".fcpxml")
-                        or item.endswith(".mlt")
-                    ):
-                        os.remove(item)
-                    if item.endswith("_tracks"):
-                        shutil.rmtree(item)
+        clean("resources")
+        clean(os.getcwd())
 
-            clean("resources")
-            clean(os.getcwd())
+    if args.only != []:
+        tests = filter(lambda t: t.__name__ in args.only, tests)
 
-        if args.only != []:
-            tests = list(filter(lambda t: t.__name__ in args.only, tests))
+    total_time = 0
 
-        for passed, test in enumerate(tests):
-            start = perf_counter()
+    for passed, test in enumerate(tests):
+        start = perf_counter()
 
-            try:
-                outputs = test()
-                end = perf_counter() - start
-            except KeyboardInterrupt:
-                print(f"Testing Interrupted by User.")
-                clean_all()
-                sys.exit(1)
-            except Exception as e:
-                print(f"Test '{test.__name__}' ({passed}/{len(tests)}) failed.\n{e}")
-                logging.error("", exc_info=True)
-                clean_all()
-                sys.exit(1)
+        try:
+            outputs = test()
+            dur = perf_counter() - start
+            total_time += dur
+        except KeyboardInterrupt:
+            print("Testing Interrupted by User.")
+            clean_all()
+            sys.exit(1)
+        except Exception as e:
+            print(f"Test '{test.__name__}' ({passed}/{len(tests)}) failed.\n{e}")
+            clean_all()
+            raise e
 
-            print(f"Test '{test.__name__}' passed: {round(end, 2)} secs")
+        print(f"{test.__name__:<25} {round(dur, 2)} secs")
 
-            if outputs is not None:
-                if isinstance(outputs, str):
-                    outputs = [outputs]
+        if outputs is not None:
+            if isinstance(outputs, str):
+                outputs = [outputs]
 
-                for out in outputs:
-                    if os.path.isfile(out):
-                        os.remove(out)
+            for out in outputs:
+                try:
+                    os.remove(out)
+                except FileNotFoundError:
+                    pass
 
-        print(f"{passed+1}/{len(tests)}")
+    print(f"\nCompleted\n{passed+1}/{len(tests)}\n{round(total_time, 2)} secs")
 
 
 def main(sys_args: list[str] | None = None):
@@ -154,6 +163,7 @@ def main(sys_args: list[str] | None = None):
     args = test_options(ArgumentParser("test")).parse_args(TestArgs, sys_args)
 
     run = Runner()
+    checker = Checker(FFmpeg(), Log())
 
     ### Tests ###
 
@@ -170,12 +180,11 @@ def main(sys_args: list[str] | None = None):
         """check the help option, its short, and help on options and groups."""
         run.raw(["--help"])
         run.raw(["-h"])
-        run.raw(["--frame_margin", "--help"])
-        run.raw(["--frame_margin", "-h"])
+        run.raw(["--margin", "--help"])
+        run.raw(["--edit", "-h"])
         run.raw(["--help", "--help"])
         run.raw(["-h", "--help"])
         run.raw(["--help", "-h"])
-        run.raw(["-h", "--help"])
 
     def version_test():
         """Test version flags and debug by itself."""
@@ -207,20 +216,41 @@ def main(sys_args: list[str] | None = None):
 
     def example():
         out = run.main(inputs=["example.mp4"], cmd=[])
-        with av.open(out) as cn:
-            video = cn.streams.video[0]
-            assert video.average_rate == 30
-            assert video.time_base == Fraction(1, 30)
-            assert video.width == 1280
-            assert video.height == 720
-            assert video.codec.name == "h264"
-            assert cn.streams.audio[0].codec.name == "aac"
-            assert cn.streams.audio[0].rate == 48000
-            assert video.language == "eng"
-            assert cn.streams.audio[0].language == "eng"
+        cn = checker.check(out)
+        video = cn.videos[0]
+
+        assert video.fps == 30
+        assert video.time_base == Fraction(1, 30)
+        assert video.width == 1280
+        assert video.height == 720
+        assert video.codec == "h264"
+        assert video.lang == "eng"
+        assert cn.audios[0].codec == "aac"
+        assert cn.audios[0].samplerate == 48000
+        assert cn.audios[0].lang == "eng"
 
         return out
 
+    def add_audio():
+        run.main(
+            ["example.mp4"],
+            [
+                "--source",
+                "snd:resources/wav/pcm-f32le.wav",
+                "--add",
+                "audio:0.3sec,end,snd,volume=0.3",
+            ],
+        )
+        return run.main(
+            ["example.mp4"],
+            [
+                "--source",
+                "snd:resources/wav/pcm-f32le.wav",
+                "--add",
+                "audio:2,40,snd,3sec",
+            ],
+        )
+
     # PR #260
     def high_speed_test():
         return run.main(inputs=["example.mp4"], cmd=["--video-speed", "99998"])
@@ -241,11 +271,13 @@ def main(sys_args: list[str] | None = None):
     def units():
         run.main(["example.mp4"], ["--mark_as_loud", "20s,22sec", "25secs,26.5seconds"])
         run.main(["example.mp4"], ["--edit", "all", "--set-speed", "125%,-30,end"])
-        run.main(["example.mp4"], ["--sample_rate", "44_100"])
         run.main(["example.mp4"], ["--margin", "3_0"])
+        return run.main(["example.mp4"], ["--edit", "audio:threshold=4%"])
+
+    def sr_units():
         run.main(["example.mp4"], ["--sample_rate", "44100 Hz"])
         run.main(["example.mp4"], ["--sample_rate", "44.1 kHz"])
-        return run.main(["example.mp4"], ["--edit", "audio:threshold=4%"])
+        return run.main(["example.mp4"], ["--sample_rate", "44_100"])
 
     def video_speed():
         return run.main(["example.mp4"], ["--video-speed", "1.5"])
@@ -279,14 +311,14 @@ def main(sys_args: list[str] | None = None):
 
     def gif():
         """
-        Feed auto-editor a gif file and make sure it can spit out a correctly formated
+        Feed auto-editor a gif file and make sure it can spit out a correctly formatted
         gif. No editing is requested.
         """
         out = run.main(
             ["resources/only-video/man-on-green-screen.gif"], ["--edit", "none"]
         )
-        with av.open(out) as cn:
-            assert cn.streams.video[0].codec.name == "gif"
+        assert checker.check(out).videos[0].codec == "gif"
+
         return out
 
     def margin_tests():
@@ -307,14 +339,13 @@ def main(sys_args: list[str] | None = None):
     def output_extension():
         # Add input extension to output name if no output extension is given.
         out = run.main(inputs=["example.mp4"], cmd=[], output="out")
+
         assert out == "out.mp4"
-        with av.open(out) as cn:
-            assert cn.streams.video[0].codec.name == "h264"
+        assert checker.check(out).videos[0].codec == "h264"
 
         out = run.main(inputs=["resources/testsrc.mkv"], cmd=[], output="out")
         assert out == "out.mkv"
-        with av.open(out) as cn:
-            assert cn.streams.video[0].codec.name == "h264"
+        assert checker.check(out).videos[0].codec == "h264"
 
         return "out.mp4", "out.mkv"
 
@@ -337,24 +368,27 @@ def main(sys_args: list[str] | None = None):
         return out, out2
 
     def resolution_and_scale():
-        with av.open(run.main(["example.mp4"], ["--scale", "1.5"])) as cn:
-            assert cn.streams.video[0].average_rate == 30
-            assert cn.streams.video[0].width == 1920
-            assert cn.streams.video[0].height == 1080
-            assert cn.streams.audio[0].rate == 48000
+        cn = checker.check(run.main(["example.mp4"], ["--scale", "1.5"]))
 
-        with av.open(run.main(["example.mp4"], ["--scale", "0.2"])) as cn:
-            assert cn.streams.video[0].average_rate == 30
-            assert cn.streams.video[0].width == 256
-            assert cn.streams.video[0].height == 144
-            assert cn.streams.audio[0].rate == 48000
+        assert cn.videos[0].fps == 30
+        assert cn.videos[0].width == 1920
+        assert cn.videos[0].height == 1080
+        assert cn.audios[0].samplerate == 48000
+
+        cn = checker.check(run.main(["example.mp4"], ["--scale", "0.2"]))
+
+        assert cn.videos[0].fps == 30
+        assert cn.videos[0].width == 256
+        assert cn.videos[0].height == 144
+        assert cn.audios[0].samplerate == 48000
 
         out = run.main(["example.mp4"], ["-res", "700,380", "-b", "darkgreen"])
-        with av.open(out) as cn:
-            assert cn.streams.video[0].average_rate == 30
-            assert cn.streams.video[0].width == 700
-            assert cn.streams.video[0].height == 380
-            assert cn.streams.audio[0].rate == 48000
+        cn = checker.check(out)
+
+        assert cn.videos[0].fps == 30
+        assert cn.videos[0].width == 700
+        assert cn.videos[0].height == 380
+        assert cn.audios[0].samplerate == 48000
 
         return out
 
@@ -364,12 +398,12 @@ def main(sys_args: list[str] | None = None):
             ["--add", "rectangle:0,30,0,0,300,300,fill=blue"],
             "out.mp4",
         )
-        with av.open(out) as cn:
-            assert len(cn.streams.video) == 1
-            assert len(cn.streams.audio) == 1
-            assert cn.streams.video[0].width == 1920
-            assert cn.streams.video[0].height == 1080
-            assert cn.streams.video[0].average_rate == 30
+        cn = checker.check(out)
+        assert len(cn.videos) == 1
+        assert len(cn.audios) == 1
+        assert cn.videos[0].width == 1920
+        assert cn.videos[0].height == 1080
+        assert cn.videos[0].fps == 30
 
         return out
 
@@ -442,7 +476,10 @@ def main(sys_args: list[str] | None = None):
             test_file = f"resources/{test_name}"
             results.add(run.main([test_file], []))
             run.main([test_file], ["--edit", "none"])
-            run.main([test_file], ["-exp"])
+
+            p_xml = run.main([test_file], ["-exp"])
+            run.main([p_xml], [])
+
             run.main([test_file], ["-exf"])
             run.main([test_file], ["-exs"])
             run.main([test_file], ["--export_as_clip_sequence"])
@@ -461,8 +498,7 @@ def main(sys_args: list[str] | None = None):
             ["--edit", "audio:stream=1"],
             "out.mov",
         )
-        with av.open(out) as cn:
-            assert len(cn.streams.audio) == 1
+        assert len(checker.check(out).audios) == 1
 
         return out
 
@@ -473,8 +509,8 @@ def main(sys_args: list[str] | None = None):
 
     def concat_mux_tracks():
         out = run.main(["example.mp4", "resources/multi-track.mov"], [], "out.mov")
-        with av.open(out) as cn:
-            assert len(cn.streams.audio) == 1
+        assert len(checker.check(out).audios) == 1
+
         return out
 
     def concat_multiple_tracks():
@@ -483,63 +519,58 @@ def main(sys_args: list[str] | None = None):
             ["--keep-tracks-separate"],
             "out.mov",
         )
-        with av.open(out) as cn:
-            assert len(cn.streams.audio) == 2, f"audio streams: {len(cn.streams.audio)}"
-
+        assert len(checker.check(out).audios) == 2
         out = run.main(
             ["example.mp4", "resources/multi-track.mov"],
             ["--keep-tracks-separate"],
             "out.mov",
         )
-        with av.open(out) as cn:
-            assert len(cn.streams.audio) == 2
+        assert len(checker.check(out).audios) == 2
 
         return out
 
     def frame_rate():
-        with av.open(run.main(["example.mp4"], ["-r", "15", "--no-seek"])) as cn:
-            video = cn.streams.video[0]
-            assert video.average_rate == 15
-            assert video.time_base == Fraction(1, 15)
-            dur = float(video.duration * video.time_base)
-            assert dur - 17.33333333333333333333333 < 3
+        cn = checker.check(run.main(["example.mp4"], ["-r", "15", "--no-seek"]))
+        video = cn.videos[0]
+        assert video.fps == 15
+        assert video.time_base == Fraction(1, 15)
+        assert float(video.duration) - 17.33333333333333333333333 < 3
 
-        with av.open(run.main(["example.mp4"], ["-r", "20"])) as cn:
-            video = cn.streams.video[0]
-            assert video.average_rate == 20
-            assert video.time_base == Fraction(1, 20)
-            dur = float(video.duration * video.time_base)
-            assert dur - 17.33333333333333333333333 < 2
+        cn = checker.check(run.main(["example.mp4"], ["-r", "20"]))
+        video = cn.videos[0]
+        assert video.fps == 20
+        assert video.time_base == Fraction(1, 20)
+        assert float(video.duration) - 17.33333333333333333333333 < 2
 
-        with av.open(out := run.main(["example.mp4"], ["-r", "60"])) as cn:
-            video = cn.streams.video[0]
-            assert video.average_rate == 60
-            assert video.time_base == Fraction(1, 60)
-            dur = float(video.duration * video.time_base)
-            assert dur - 17.33333333333333333333333 < 0.3
+        cn = checker.check(out := run.main(["example.mp4"], ["-r", "60"]))
+        video = cn.videos[0]
+
+        assert video.fps == 60
+        assert video.time_base == Fraction(1, 60)
+        assert float(video.duration) - 17.33333333333333333333333 < 0.3
 
         return out
 
-    def image():
+    def embedded_image():
         out1 = run.main(["resources/embedded-image/h264-png.mp4"], [])
-        with av.open(out1) as cn:
-            assert cn.streams.video[0].codec.name == "h264"
-            assert cn.streams.video[1].codec.name == "png"
+        cn = checker.check(out1)
+        assert cn.videos[0].codec == "h264"
+        assert cn.videos[1].codec == "png"
 
         out2 = run.main(["resources/embedded-image/h264-mjpeg.mp4"], [])
-        with av.open(out2) as cn:
-            assert cn.streams.video[0].codec.name == "h264"
-            assert cn.streams.video[1].codec.name == "mjpeg"
+        cn = checker.check(out2)
+        assert cn.videos[0].codec == "h264"
+        assert cn.videos[1].codec == "mjpeg"
 
         out3 = run.main(["resources/embedded-image/h264-png.mkv"], [])
-        with av.open(out3) as cn:
-            assert cn.streams.video[0].codec.name == "h264"
-            assert cn.streams.video[1].codec.name == "png"
+        cn = checker.check(out3)
+        assert cn.videos[0].codec == "h264"
+        assert cn.videos[1].codec == "png"
 
         out4 = run.main(["resources/embedded-image/h264-mjpeg.mkv"], [])
-        with av.open(out4) as cn:
-            assert cn.streams.video[0].codec.name == "h264"
-            assert cn.streams.video[1].codec.name == "mjpeg"
+        cn = checker.check(out4)
+        assert cn.videos[0].codec == "h264"
+        assert cn.videos[1].codec == "mjpeg"
 
         return out1, out2, out3, out4
 
@@ -557,11 +588,14 @@ def main(sys_args: list[str] | None = None):
     def edit_positive_tests():
         run.main(["resources/multi-track.mov"], ["--edit", "audio:stream=all"])
         run.main(["resources/multi-track.mov"], ["--edit", "not audio:stream=all"])
+        run.main(
+            ["resources/multi-track.mov"],
+            ["--edit", "(or (not audio:threshold=4%) audio:stream=1)"],
+        )
         out = run.main(
             ["resources/multi-track.mov"],
-            ["--edit", "not audio:threshold=4% or audio:stream=1"],
+            ["--edit", "(or (not audio:threshold=4%) (not audio:stream=1))"],
         )
-        # '--edit', 'not audio:threshold=4% or not audio:stream=1'
         return out
 
     def edit_negative_tests():
@@ -573,30 +607,6 @@ def main(sys_args: list[str] | None = None):
             ["resources/only-video/man-on-green-screen.gif", "--edit", "audio"],
             "Audio stream '0' does not exist",
         )
-        run.check(["example.mp4", "--edit", "not"], "Error! Dangling operand: 'not'")
-        run.check(
-            ["example.mp4", "--edit", "audio and"], "Error! Dangling operand: 'and'"
-        )
-        run.check(
-            ["example.mp4", "--edit", "and"],
-            "Error! 'and' operand needs two arguments.",
-        )
-        run.check(
-            ["example.mp4", "--edit", "and audio"],
-            "Error! 'and' operand needs two arguments.",
-        )
-        run.check(
-            ["example.mp4", "--edit", "or audio"],
-            "Error! 'or' operand needs two arguments.",
-        )
-        run.check(
-            ["example.mp4", "--edit", "audio four audio"],
-            "Error! Unknown method/operator: 'four'",
-        )
-        run.check(
-            ["example.mp4", "--edit", "audio audio"],
-            "Logic operator must be between two editing methods",
-        )
 
     def yuv442p():
         return run.main(["resources/test_yuv422p.mp4"], [])
@@ -604,12 +614,135 @@ def main(sys_args: list[str] | None = None):
     #  Issue 280
     def SAR():
         out = run.main(["resources/SAR-2by3.mp4"], [])
-        with av.open(out) as cn:
-            assert cn.streams.video[0].sample_aspect_ratio == Fraction(2, 3)
+        assert checker.check(out).videos[0].sar == "2:3"
+
         return out
 
+    def inter():
+        def my_try(text: str, expected: Any) -> Any:
+            try:
+                lexer = Lexer(text)
+                parser = Parser(lexer)
+                interpreter = Interpreter(parser, None)
+                interpreter.GLOBAL_SCOPE["timebase"] = Fraction(30)
+                results = interpreter.interpret()
+            except MyError as e:
+                raise ValueError(f"{text}\nMyError: {e}")
+
+            if isinstance(expected, np.ndarray):
+                if not np.array_equal(expected, results[-1]):
+                    raise ValueError(f"{text}: Numpy arrays don't match")
+            elif expected != results[-1]:
+                raise ValueError(f"{text}: Expected: {expected}, got {results[-1]}")
+
+        my_try("345", 345)
+        my_try("238.5", 238.5)
+        my_try("-34", -34)
+        my_try("-98.3", -98.3)
+        my_try("+3i", 3j)
+        my_try("3sec", 90)
+        my_try("-3sec", -90)
+        my_try("0.2sec", 6)
+        my_try("(+ 4 3)", 7)
+        my_try("(+ 4 3 2)", 9)
+        my_try("(+ 10.5 3)", 13.5)
+        my_try("(+ 3+4i -2-2i)", 1 + 2j)
+        my_try("(+ 3+4i -2-2i 5)", 6 + 2j)
+        my_try("(- 4 3)", 1)
+        my_try("(- 3)", -3)
+        my_try("(- 10.5 3)", 7.5)
+        my_try("(* 11.5 3)", 34.5)
+        my_try("(/ 3/4 4)", Fraction(3, 16))
+        my_try("(/ 5)", Fraction(1, 5))
+        my_try("(/ 6 1)", 6)
+        my_try("(sqrt -4)", 2j)
+        my_try("(expt 2 3)", 8)
+        my_try("(expt 4 0.5)", 2.0)
+        my_try("(abs 1.0)", 1.0)
+        my_try("(abs -1)", 1)
+        my_try("(boolean? -4)", False)
+        my_try("(boolean? false)", True)
+        my_try("(boolean? #t)", True)
+        my_try("(integer? 3.0)", True)
+        my_try("(integer? 4/5)", False)
+        my_try("(exact? 3.0)", False)
+        my_try("(exact? 3)", True)
+        my_try('(string-append "Hello" " World")', "Hello World")
+        my_try('(define apple "Red Wood") apple', "Red Wood")
+        my_try("(= 1 1.0)", True)
+        my_try("(= 1 2)", False)
+        my_try("(= 2+3i 2+3i 2+3i)", True)
+        my_try("(= 1)", True)
+        my_try("(+)", 0)
+        my_try("(*)", 1)
+        my_try('(define num 13) ; Set number to 13\n"Hello"', "Hello")
+        my_try('(if #t "Hello" apple)', "Hello")
+        my_try('(if #f mango "Hi")', "Hi")
+        my_try('{if (= [+ 3 4] 7) "yes" "no"}', "yes")
+        my_try("((if #t + -) 3 4)", 7)
+        my_try("((if #t + oops) 3+3i 4-2i)", 7 + 1j)
+        my_try("((if #f + -) 3 4)", -1)
+        my_try("(when (positive? 3) 17)", 17)
+        my_try("(string)", "")
+        my_try("(string #\\a)", "a")
+        my_try("(string #\\a #\\b)", "ab")
+        my_try("(string #\\a #\\b #\\c)", "abc")
+        my_try(
+            "(margin 0 (boolarr 0 0 0 1 0 0 0))",
+            np.array([0, 0, 0, 1, 0, 0, 0], dtype=np.bool_),
+        )
+        my_try(
+            "(margin -2 2 (boolarr 0 0 1 1 0 0 0))",
+            np.array([0, 0, 0, 0, 1, 1, 0], dtype=np.bool_),
+        )
+        my_try("(count-nonzero (boolarr 0 0 1 1 0 1))", 3)
+        my_try("(equal? 3 3)", True)
+        my_try("(equal? 3 3.0)", False)
+        my_try('(equal? 16.3 "Editor")', False)
+        my_try("(equal? (boolarr 1 1 0) (boolarr 1 1 0))", True)
+        my_try("(equal? (boolarr 0 1 0) (boolarr 1 1 0))", False)
+        my_try("(equal? (boolarr 0 1 0) (boolarr 0 1 0 0))", False)
+        my_try("(equal? #\\a #\\a)", True)
+        my_try('(equal? "a" #\\a)', False)
+        my_try("(equal? (list 1 2 3) (vector 1 2 3))", False)
+        my_try("(equal? (vector 1 2 3) (vector 1 2 3))", True)
+        my_try("(equal? (list 1 2 3) (list 1 2 3))", True)
+        my_try("(equal? (list 1 2 3) (cons 1 (cons 2 (cons 3 '()))))", True)
+        my_try("(equal? (list 1 2 3) (list 1 2 4))", False)
+        my_try(
+            "(or (boolarr 1 0 0) (boolarr 0 0 0 1))",
+            np.array([1, 0, 0, 1], dtype=np.bool_),
+        )
+
+        my_try("(list 1 2 3)", ConsType(1, ConsType(2, ConsType(3, Null()))))
+        my_try("(list-ref (list 0 10 20) 2)", 20)
+        my_try("(list-ref (list 0 10 20) 0)", 0)
+        my_try("(car (cons 3 4))", 3)
+        my_try("(car (list 3 4))", 3)
+        my_try("(cdr (cons 3 4))", 4)
+        my_try("(cdr (list 3 4))", ConsType(4, Null()))
+        my_try("'()", Null())
+        my_try("(length (list 1 2 3))", 3)
+        my_try("(length (list))", 0)
+        my_try("(length (boolarr 0 1 0))", 3)
+        my_try("(begin)", None)
+        my_try("(begin (define r 10) (* pi (* r r)))", 314.1592653589793)
+
+        my_try("(quote ())", Null())
+        my_try("(quote (3))", ConsType(3, Null()))
+        my_try(
+            '(quote (3 2 "apple"))', ConsType(3, ConsType(2, ConsType("apple", Null())))
+        )
+        my_try("(quote +3i)", 3j)
+        my_try("(quote 23.4)", 23.4)
+        my_try('(quote "hello")', "hello")
+        my_try("(for/vector ([i (vector 0 1 2)]) i)", [0, 1, 2])
+
     tests = []
 
+    if args.category in ("unit", "all"):
+        tests.append(inter)
+
     if args.category in ("api", "all"):
         tests.append(read_api_0_1)
 
@@ -619,6 +752,7 @@ def main(sys_args: list[str] | None = None):
     if args.category in ("cli", "all"):
         tests.extend(
             [
+                add_audio,
                 video_speed,
                 expand_chunks,
                 SAR,
@@ -643,9 +777,10 @@ def main(sys_args: list[str] | None = None):
                 export,
                 high_speed_test,
                 units,
+                sr_units,
                 backwards_range,
                 cut_out,
-                image,
+                embedded_image,
                 gif,
                 margin_tests,
                 input_extension,
@@ -659,7 +794,7 @@ def main(sys_args: list[str] | None = None):
             ]
         )
 
-    Tester().run(tests, args)
+    run_tests(tests, args)
 
 
 if __name__ == "__main__":
